@@ -2,14 +2,23 @@ import os
 import pandas
 import selenium
 from time import sleep
-from .helper import Helper
+from helper import Helper
+# from .helper import Helper
 from selenium import webdriver
 import geckodriver_autoinstaller
-geckodriver_autoinstaller.install()
 
 
 class Content:
 
+    try:
+        geckodriver_autoinstaller.install()
+    except FileNotFoundError:
+        Helper.logo_msg()
+        print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+        print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+        exit()
+
+
     def absolute_bytes_difference(wiki, period, filter, interval):
         try:
 
@@ -66,21 +75,31 @@ class Content:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -89,18 +108,23 @@ class Content:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def edited_pages(wiki, period, filter, interval):
         try:
-            
+
             try:
                 wikis = Helper.get_Wikis_Codes()
             except NameError:
@@ -156,21 +180,31 @@ class Content:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -179,18 +213,23 @@ class Content:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def net_bytes_difference(wiki, period, filter, interval):
         try:
-            
+
             try:
                 wikis = Helper.get_Wikis_Codes()
             except NameError:
@@ -244,21 +283,31 @@ class Content:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -267,18 +316,23 @@ class Content:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def pages_to_date(wiki, period, filter, interval):
         try:
-            
+
             try:
                 wikis = Helper.get_Wikis_Codes()
             except NameError:
@@ -292,7 +346,7 @@ class Content:
                        'editor_type~group-bot', 'editor-type-all':'editor_type~anonymous*group-bot*name-bot*user'}
 
             intervals = {'daily':'daily', 'monthly':'monthly'}
-            
+
             options = webdriver.FirefoxOptions()
             options.add_argument("--headless")
             profile = webdriver.FirefoxProfile()
@@ -332,21 +386,31 @@ class Content:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -355,13 +419,18 @@ class Content:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def total_media_requests(wiki, period, filter, interval):
@@ -420,21 +489,31 @@ class Content:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -443,18 +522,23 @@ class Content:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
 
 
     def top_media_requests(wiki, period, filter, interval):
         try:
-            
+
             try:
                 wikis = Helper.get_Wikis_Codes()
             except NameError:
@@ -510,21 +594,31 @@ class Content:
             print(f"\u001b[91mError:\u001b[0m exiting due to pressing ctrl-c ...\n")
             exit()
 
+        except FileNotFoundError:
+            print(f"\u001b[91mError:\u001b[0m Firefox web browser application is not installed on this machine.")
+            print("       To download the latest version of Firefox: https://www.mozilla.org/en-US/firefox/new/ \n")
+            exit()
+
+        except OSError:
+            print(f"\u001b[91mError:\u001b[0m seems `geckodriver-autoinstaller' downloaded a bad or incompatible version of the executable file of `geckodriver`.")
+            print("       Please, download the compatible `geckodriver` manually from here: https://github.com/mozilla/geckodriver/releases \n")
+            exit()
+
         except KeyError:
             if wiki not in wikis:
                 print(f"\u001b[91mError:\u001b[0m this Wikipedia: '{wiki}' is not supported. To list Wikipedias with codes: `wikistats2csv -lw/--list-wikis`.\n")
             elif period not in periods:
                 print(f"\u001b[91mError:\u001b[0m this time period: '{period}' is not supported. See GitHub for details: ")
-                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")            
+                print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif filter not in filters:
                 print(f"\u001b[91mError:\u001b[0m this filter: '{filter}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
             elif interval not in intervals:
                 print(f"\u001b[91mError:\u001b[0m this time interval: '{interval}' is not supported. See GitHub for details: ")
                 print("       https://github.com/SaiedAlshahrani/Wikistats-to-CSV/blob/main/README.md \n")
-            else: 
+            else:
                 print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
-        
+
         except selenium.common.exceptions.WebDriverException:
             print(f"\u001b[91mError:\u001b[0m 'geckodriver' executable needs to be in PATH. See this documentation for details: ")
             print("       https://www.selenium.dev/documentation/webdriver/getting_started/install_drivers/ \n")
@@ -533,10 +627,15 @@ class Content:
             print(f"\u001b[91mError:\u001b[0m one of these parameters: period-->'{period}', filter-->'{filter}', or interval-->'{interval}' is not supported.\n")
 
         except:
-            if 'Loading metric...' in html:
-                print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
-                print(request_url, '\n')
-            elif 'There is no data available for this date range on this project' in html:
-                print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
-            else: 
+            try:
+                if 'Loading metric...' in html:
+                    print(f"\u001b[91mError:\u001b[0m cannot load and save this metric due to connection timeout!! Try again, or access it manually from here:")
+                    print(request_url, '\n')
+
+                elif 'There is no data available for this date range on this project' in html:
+                    print("\u001b[91mError:\u001b[0m there is no data available for this date range on this Wikipedia.\n")
+                else:
+                    print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+            except UnboundLocalError:
                 print("\u001b[91mError:\u001b[0m something unknown went wrong!! Please, try again!!\n")
+                pass
